@@ -18,6 +18,8 @@
     CouchIndexPage: function() {
       page = this;
 
+      $.futon.storage.declare("per_page", {defaultValue: 10});
+
       this.addDatabase = function() {
         $.showDialog("dialog/_create_database.html", {
           submit: function(data, callback) {
@@ -60,7 +62,8 @@
                     .find("td.size").text($.futon.formatSize(info.disk_size)).end()
                     .find("td.count").text(info.doc_count).end()
                     .find("td.seq").text(info.update_seq);
-                }
+                },
+                error : function() {}
               });
             });
             $("#databases tbody tr:odd").addClass("odd");
@@ -95,11 +98,24 @@
     CouchDatabasePage: function() {
       var urlParts = location.search.substr(1).split("/");
       var dbName = decodeURIComponent(urlParts.shift());
+
+      $.futon.storage.declareWithPrefix(dbName + ".", {
+        desc: {},
+        language: {defaultValue: "javascript"},
+        map_fun: {defaultValue: ""},
+        reduce_fun: {defaultValue: ""},
+        reduce: {},
+        group_level: {defaultValue: 100},
+        per_page: {defaultValue: 10},
+        view: {defaultValue: ""},
+        stale: {defaultValue: false}
+      });
+
       var viewName = (urlParts.length > 0) ? urlParts.join("/") : null;
       if (viewName) {
-        $.cookies.set(dbName + ".view", viewName);
+        $.futon.storage.set("view", viewName);
       } else {
-        viewName = $.cookies.get(dbName + ".view", "");
+        viewName = $.futon.storage.get("view");
         if (viewName) {
           this.redirecting = true;
           location.href = "database.html?" + encodeURIComponent(dbName) +
@@ -119,21 +135,29 @@
       var templates = {
         javascript: "function(doc) {\n  emit(null, doc);\n}",
         python: "def fun(doc):\n  yield None, doc",
-        ruby: "{|doc|\n  emit(nil, doc);\n}"
+        ruby: "lambda {|doc|\n  emit(nil, doc);\n}"
       }
 
       this.newDocument = function() {
         location.href = "document.html?" + encodeURIComponent(db.name);
       }
 
-      this.compactDatabase = function() {
-        $.showDialog("dialog/_compact_database.html", {
+      this.compactAndCleanup = function() {
+        $.showDialog("dialog/_compact_cleanup.html", {
           submit: function(data, callback) {
-            db.compact({
-              success: function(resp) {
-                callback();
-              }
-            });
+            switch (data.action) {
+              case "compact_database":
+                db.compact({success: function(resp) { callback() }});
+                break;
+              case "compact_views":
+                var groupname = page.viewName.substring(8,
+                    page.viewName.indexOf("/_view"));
+                db.compactView(groupname, {success: function(resp) { callback() }});
+                break;
+              case "view_cleanup":
+                db.viewCleanup({success: function(resp) { callback() }});
+                break;
+            }
           }
         });
       }
@@ -157,6 +181,76 @@
         });
       }
 
+      this.databaseSecurity = function() {
+        $.showDialog("dialog/_database_security.html", {
+          load : function(d) {
+            db.getDbProperty("_security", {
+              success: function(r) {
+                ["admin", "reader"].forEach(function(key) {
+                  var names = [];
+                  var roles = [];
+
+                  if (r && typeof r[key + "s"] === "object") {
+                    if ($.isArray(r[key + "s"]["names"])) {
+                      names = r[key + "s"]["names"];
+                    }
+                    if ($.isArray(r[key + "s"]["roles"])) {
+                      roles = r[key + "s"]["roles"];
+                    }
+                  }
+
+                  $("input[name=" + key + "_names]", d).val(JSON.stringify(names));
+                  $("input[name=" + key + "_roles]", d).val(JSON.stringify(roles));
+                });
+              }
+            });
+          },
+          // maybe this should be 2 forms
+          submit: function(data, callback) {
+            var errors = {};
+            var secObj = {
+              admins: {
+                names: [],
+                roles: []
+              },
+              readers: {
+                names: [],
+                roles: []
+              }
+            };
+
+            ["admin", "reader"].forEach(function(key) {
+              var names, roles;
+
+              try {
+                names = JSON.parse(data[key + "_names"]);
+              } catch(e) { }
+              try {
+                roles = JSON.parse(data[key + "_roles"]);
+              } catch(e) { }
+
+              if ($.isArray(names)) {
+                secObj[key + "s"]["names"] = names;
+              } else {
+                errors[key + "_names"] = "The " + key +
+                  " names must be an array of strings";
+              }
+              if ($.isArray(roles)) {
+                secObj[key + "s"]["roles"] = roles;
+              } else {
+                errors[key + "_roles"] = "The " + key +
+                  " roles must be an array of strings";
+              }
+            });
+
+            if ($.isEmptyObject(errors)) {
+              db.setDbProperty("_security", secObj);
+            }
+            callback(errors);
+          }
+        });
+      }
+
       this.populateViewEditor = function() {
         if (viewName.match(/^_design\//)) {
           page.revertViewChanges(function() {
@@ -165,8 +259,13 @@
               clearTimeout(dirtyTimeout);
               dirtyTimeout = setTimeout(function() {
                 var buttons = $("#viewcode button.save, #viewcode button.revert");
-                page.isDirty = ($("#viewcode_map").val() != page.storedViewCode.map)
-                  || ($("#viewcode_reduce").val() != (page.storedViewCode.reduce || ""))
+                var viewCode = {
+                  map: $("#viewcode_map").val(),
+                  reduce: $("#viewcode_reduce").val()
+                };
+                $("#reduce, #grouplevel").toggle(!!viewCode.reduce);
+                page.isDirty = (viewCode.map != page.storedViewCode.map)
+                  || (viewCode.reduce != (page.storedViewCode.reduce || ""))
                   || page.viewLanguage != page.storedViewLanguage;
                 if (page.isDirty) {
                   buttons.removeAttr("disabled");
@@ -175,7 +274,8 @@
                 }
               }, 100);
             }
-            $("#viewcode textarea").bind("input", updateDirtyState);
+            $("#viewcode textarea").enableTabInsertion()
+              .bind("input", updateDirtyState);
             if ($.browser.msie || $.browser.safari) {
               $("#viewcode textarea").bind("paste", updateDirtyState)
                                      .bind("change", updateDirtyState)
@@ -187,15 +287,15 @@
             $("#language").change(updateDirtyState);
             page.updateDocumentListing();
           });
-          $("#grouptruenotice").show();
         } else if (viewName == "_temp_view") {
-          page.viewLanguage = $.cookies.get(db.name + ".language", page.viewLanguage);
+          $("#viewcode textarea").enableTabInsertion();
+          page.viewLanguage = $.futon.storage.get("language");
           page.updateViewEditor(
-            $.cookies.get(db.name + ".map", templates[page.viewLanguage]),
-            $.cookies.get(db.name + ".reduce", "")
+            $.futon.storage.get("map_fun", templates[page.viewLanguage]),
+            $.futon.storage.get("reduce_fun")
           );
-          $("#grouptruenotice").show();
         } else {
+          $("#grouplevel, #reduce").hide();
           page.updateDocumentListing();
         }
         page.populateLanguagesMenu();
@@ -248,7 +348,8 @@
                 fill_language();
               }
             }, "native_query_servers");
-          }
+          },
+          error : function() {}
         }, "query_servers");
       }
 
@@ -296,13 +397,13 @@
           db.openDoc("_design/" + designDocId, {
             error: function(status, error, reason) {
               if (status == 404) {
-                $.cookies.remove(dbName + ".view");
+                $.futon.storage.del("view");
                 location.href = "database.html?" + encodeURIComponent(db.name);
               }
             },
             success: function(resp) {
               if(!resp.views || !resp.views[localViewName]) {
-                $.cookies.remove(dbName + ".view");
+                $.futon.storage.del("view");
                 location.href = "database.html?" + encodeURIComponent(db.name);
               }
               var viewCode = resp.views[localViewName];
@@ -314,7 +415,7 @@
               page.storedViewLanguage = page.viewLanguage;
               if (callback) callback();
             }
-          },{async:false});
+          }, {async: false});
         } else {
           page.updateViewEditor(page.storedViewCode.map,
             page.storedViewCode.reduce || "");
@@ -334,6 +435,7 @@
           mapFun.split("\n").length,
           reduceFun.split("\n").length
         );
+        $("#reduce, #grouplevel").toggle(!!reduceFun);
         $("#viewcode textarea").attr("rows", Math.min(15, Math.max(3, lines)));
       }
 
@@ -485,41 +587,41 @@
       this.updateDocumentListing = function(options) {
         if (options === undefined) options = {};
         if (options.limit === undefined) {
-          per_page = parseInt($("#perpage").val(), 10)
+          var perPage = parseInt($("#perpage").val(), 10)
           // Fetch an extra row so we know when we're on the last page for
           // reduce views
-          options.limit = per_page + 1;
+          options.limit = perPage + 1;
         } else {
-          per_page = options.limit - 1;
+          perPage = options.limit - 1;
         }
         if ($("#documents thead th.key").is(".desc")) {
           if (typeof options.descending == 'undefined') options.descending = true;
           var descend = true;
-          $.cookies.set(dbName + ".desc", "1");
+          $.futon.storage.set("desc", "1");
         } else {
           var descend = false;
-          $.cookies.remove(dbName + ".desc");
+          $.futon.storage.del("desc");
         }
         $("#paging a").unbind();
         $("#documents").find("tbody.content").empty().end().show();
-        this.updateDesignDocLink();
+        page.updateDesignDocLink();
 
         options.success = function(resp) {
           if (resp.offset === undefined) {
             resp.offset = 0;
           }
           var descending_reverse = ((options.descending && !descend) || (descend && (options.descending === false)));
-          var has_reduce_prev = resp.total_rows === undefined && (descending_reverse ? resp.rows.length > per_page : options.startkey !== undefined);
+          var has_reduce_prev = resp.total_rows === undefined && (descending_reverse ? resp.rows.length > perPage : options.startkey !== undefined);
           if (descending_reverse && resp.rows) {
             resp.rows = resp.rows.reverse();
-            if (resp.rows.length > per_page) {
+            if (resp.rows.length > perPage) {
               resp.rows.push(resp.rows.shift());
             }
           }
           if (resp.rows !== null && (has_reduce_prev || (descending_reverse ?
-            (resp.total_rows - resp.offset > per_page) :
+            (resp.total_rows - resp.offset > perPage) :
             (resp.offset > 0)))) {
-            $("#paging a.prev").attr("href", "#" + (resp.offset - per_page)).click(function() {
+            $("#paging a.prev").attr("href", "#" + (resp.offset - perPage)).click(function() {
               var opt = {
                 descending: !descend,
                 limit: options.limit
@@ -538,17 +640,17 @@
           } else {
             $("#paging a.prev").removeAttr("href");
           }
-          var has_reduce_next = resp.total_rows === undefined && (descending_reverse ? options.startkey !== undefined : resp.rows.length > per_page);
+          var has_reduce_next = resp.total_rows === undefined && (descending_reverse ? options.startkey !== undefined : resp.rows.length > perPage);
           if (resp.rows !== null && (has_reduce_next || (descending_reverse ?
-            (resp.offset - resp.total_rows < per_page) :
-            (resp.total_rows - resp.offset > per_page)))) {
-            $("#paging a.next").attr("href", "#" + (resp.offset + per_page)).click(function() {
+            (resp.offset - resp.total_rows < perPage) :
+            (resp.total_rows - resp.offset > perPage)))) {
+            $("#paging a.next").attr("href", "#" + (resp.offset + perPage)).click(function() {
               var opt = {
                 descending: descend,
                 limit: options.limit
               };
               if (resp.rows.length > 0) {
-                var lastDoc = resp.rows[Math.min(per_page, resp.rows.length) - 1];
+                var lastDoc = resp.rows[Math.min(perPage, resp.rows.length) - 1];
                 opt.startkey = lastDoc.key !== undefined ? lastDoc.key : null;
                 if (lastDoc.id !== undefined) {
                   opt.startkey_docid = lastDoc.id;
@@ -562,7 +664,7 @@
             $("#paging a.next").removeAttr("href");
           }
 
-          for (var i = 0; i < Math.min(per_page, resp.rows.length); i++) {
+          for (var i = 0; i < Math.min(perPage, resp.rows.length); i++) {
             var row = resp.rows[i];
             var tr = $("<tr></tr>");
             var key = "null";
@@ -572,7 +674,7 @@
             if (row.id) {
               $("<td class='key'><a href='document.html?" + encodeURIComponent(db.name) +
                 "/" + $.couch.encodeDocId(row.id) + "'><strong></strong><br>" +
-                "<span class='docid'>ID:&nbsp;" + row.id + "</span></a></td>")
+                "<span class='docid'>ID:&nbsp;" + $.futon.escape(row.id) + "</span></a></td>")
                 .find("strong").text(key).end()
                 .appendTo(tr);
             } else {
@@ -586,13 +688,14 @@
                 html: true, indent: 0, linesep: "", quoteKeys: false
               });
             }
-            $("<td class='value'><div></div></td>").find("div").html(value).end().appendTo(tr).dblclick(function() {
-              location.href = this.previousSibling.firstChild.href;
-            });
+            $("<td class='value'><div></div></td>").find("div").html(value).end()
+              .appendTo(tr).dblclick(function() {
+                location.href = this.previousSibling.firstChild.href;
+              });
             tr.appendTo("#documents tbody.content");
           }
           var firstNum = 1;
-          var lastNum = totalNum = Math.min(per_page, resp.rows.length);
+          var lastNum = totalNum = Math.min(perPage, resp.rows.length);
           if (resp.total_rows != null) {
             if (descending_reverse) {
               lastNum = Math.min(resp.total_rows, resp.total_rows - resp.offset);
@@ -623,31 +726,51 @@
           if (viewName == "_temp_view") {
             $("#viewcode").show().removeClass("collapsed");
             var mapFun = $("#viewcode_map").val();
-            $.cookies.set(db.name + ".map", mapFun);
+            $.futon.storage.set("map_fun", mapFun);
             var reduceFun = $.trim($("#viewcode_reduce").val()) || null;
             if (reduceFun) {
-              $.cookies.set(db.name + ".reduce", reduceFun);
-              options.group = true;
-            } else {
-              $.cookies.remove(db.name + ".reduce");
+              $.futon.storage.set("reduce_fun", reduceFun);
+              if ($("#reduce :checked").length) {
+                var level = parseInt($("#grouplevel select").val(), 10);
+                options.group = level > 0;
+                if (options.group && level < 100) {
+                  options.group_level = level;
+                }
+              } else {
+                options.reduce = false;
+              }
             }
-            $.cookies.set(db.name + ".language", page.viewLanguage);
+            $.futon.storage.set("language", page.viewLanguage);
             db.query(mapFun, reduceFun, page.viewLanguage, options);
           } else if (viewName == "_design_docs") {
             options.startkey = options.descending ? "_design0" : "_design";
             options.endkey = options.descending ? "_design" : "_design0";
             db.allDocs(options);
           } else {
+            $("button.compactview").show();
             $("#viewcode").show();
             var currentMapCode = $("#viewcode_map").val();
             var currentReduceCode = $.trim($("#viewcode_reduce").val()) || null;
             if (currentReduceCode) {
-              options.group = true;
+              if ($("#reduce :checked").length) {
+                var level = parseInt($("#grouplevel select").val(), 10);
+                options.group = level > 0;
+                if (options.group && level < 100) {
+                  options.group_level = level;
+                }
+              } else {
+                options.reduce = false;
+              }
             }
             if (page.isDirty) {
               db.query(currentMapCode, currentReduceCode, page.viewLanguage, options);
             } else {
               var viewParts = viewName.split('/');
+
+              if ($.futon.storage.get("stale")) {
+                 options.stale = "ok";
+              }
+
               db.view(viewParts[1] + "/" + viewParts[3], options);
             }
           }
@@ -664,7 +787,7 @@
 
     },
 
-    // Page class for browse/database.html
+    // Page class for browse/document.html
     CouchDocumentPage: function() {
       var urlParts = location.search.substr(1).split("/");
       var dbName = decodeURIComponent(urlParts.shift());
@@ -680,6 +803,8 @@
       }
       var db = $.couch.db(dbName);
 
+      $.futon.storage.declare("tab", {defaultValue: "tabular", scope: "cookie"});
+
       this.dbName = dbName;
       this.db = db;
       this.docId = docId;
@@ -691,13 +816,16 @@
         if ($("#fields tbody.source textarea").length > 0)
           return;
 
+        $.futon.storage.set("tab", "tabular");
         $("#tabs li").removeClass("active").filter(".tabular").addClass("active");
         $("#fields thead th:first").text("Field").attr("colspan", 1).next().show();
         $("#fields tbody.content").show();
         $("#fields tbody.source").hide();
+        return false;
       }
 
       this.activateSourceView = function() {
+        $.futon.storage.set("tab", "source");
         $("#tabs li").removeClass("active").filter(".source").addClass("active");
         $("#fields thead th:first").text("Source").attr("colspan", 2).next().hide();
         $("#fields tbody.content").hide();
@@ -705,7 +833,8 @@
           $(this).html($("<pre></pre>").html($.futon.formatJSON(page.doc, {html: true})))
             .makeEditable({allowEmpty: false,
               createInput: function(value) {
-                return $("<textarea rows='8' cols='80' spellcheck='false'></textarea>");
+                var rows = value.split("\n").length;
+                return $("<textarea rows='" + rows + "' cols='80' spellcheck='false'></textarea>").enableTabInsertion();
               },
               prepareInput: function(input) {
                 $(input).makeResizable({vertical: true});
@@ -729,7 +858,7 @@
                   return true;
                 } catch (err) {
                   var msg = err.message;
-                  if (msg == "parseJSON" || msg == "JSON.parse") { 
+                  if (msg == "parseJSON" || msg == "JSON.parse") {
                     msg = "There is a syntax error in the document.";
                   }
                   $("<div class='error'></div>").text(msg).appendTo(this);
@@ -738,6 +867,7 @@
               }
             });
         }).end().show();
+        return false;
       }
 
       this.addField = function() {
@@ -804,7 +934,7 @@
             $("#fields tbody.footer td span").text("Showing revision " +
               (revs.length - currentIndex) + " of " + revs.length);
           }
-          if (location.hash == "#source" && !noReload) {
+          if ($.futon.storage.get("tab") == "source") {
             page.activateSourceView();
           }
         }
@@ -970,10 +1100,14 @@
           return;
         }
 
-        row.find("td").makeEditable({allowEmpty: true,
+        row.find("td").makeEditable({acceptOnBlur: false, allowEmpty: true,
           createInput: function(value) {
-            if ($("dl", this).length > 0 || $("code", this).text().length > 60) {
-              return $("<textarea rows='8' cols='40' spellcheck='false'></textarea>");
+            value = doc[row.data("name")];
+            var elem = $(this);
+            if (elem.find("dl").length > 0 ||
+                elem.find("code").is(".array, .object") ||
+                typeof(value) == "string" && (value.length > 60 || value.match(/\n/))) {
+              return $("<textarea rows='1' cols='40' spellcheck='false'></textarea>");
             }
             return $("<input type='text' spellcheck='false'>");
           },
@@ -983,12 +1117,17 @@
           },
           prepareInput: function(input) {
             if ($(input).is("textarea")) {
-              $(input).makeResizable({vertical: true});
+              var height = Math.min(input.scrollHeight, document.body.clientHeight - 100);
+              $(input).height(height).makeResizable({vertical: true}).enableTabInsertion();
             }
           },
           accept: function(newValue) {
             var fieldName = row.data("name");
-            doc[fieldName] = JSON.parse(newValue);
+            try {
+              doc[fieldName] = JSON.parse(newValue);
+            } catch (err) {
+              doc[fieldName] = newValue;
+            }
             page.isDirty = true;
             if (fieldName == "_id") {
               page.docId = page.doc._id = doc[fieldName];
@@ -996,7 +1135,11 @@
             }
           },
           populate: function(value) {
-            return $.futon.formatJSON(doc[row.data("name")]);
+            value = doc[row.data("name")];
+            if (typeof(value) == "string") {
+              return value;
+            }
+            return $.futon.formatJSON(value);
           },
           validate: function(value) {
             $("div.error", this).remove();
@@ -1009,21 +1152,21 @@
               }
               return true;
             } catch (err) {
-              var msg = err.message;
-              if (msg == "parseJSON" || msg == "JSON.parse") {
-                msg = "Please enter a valid JSON value (for example, \"string\").";
-              }
-              $("<div class='error'></div>").text(msg).appendTo(this);
-              return false;
+              return true;
             }
           }
         });
       }
 
       function _renderValue(value) {
+        function isNullOrEmpty(val) {
+          if (val == null) return true;
+          for (var i in val) return false;
+          return true;
+        }
         function render(val) {
           var type = typeof(val);
-          if (type == "object" && val !== null) {
+          if (type == "object" && !isNullOrEmpty(val)) {
             var list = $("<dl></dl>");
             for (var i in val) {
               $("<dt></dt>").text(i).appendTo(list);
@@ -1031,7 +1174,44 @@
             }
             return list;
           } else {
-            return $($.futon.formatJSON(val, {html: true}));
+            var html = $.futon.formatJSON(val, {
+              html: true,
+              escapeStrings: false
+            });
+            var n = $(html);
+            if (n.text().length > 140) {
+              // This code reduces a long string in to a summarized string with a link to expand it.
+              // Someone, somewhere, is doing something nasty with the event after it leaves these handlers.
+              // At this time I can't track down the offender, it might actually be a jQuery propogation issue.
+              var fulltext = n.text();
+              var mintext = n.text().slice(0, 140);
+              var e = $('<a href="#expand">...</a>');
+              var m = $('<a href="#min">X</a>');
+              var expand = function (evt) {
+                n.empty();
+                n.text(fulltext);
+                n.append(m);
+                evt.stopPropagation();
+                evt.stopImmediatePropagation();
+                evt.preventDefault();
+              }
+              var minimize = function (evt) {
+                n.empty();
+                n.text(mintext);
+                // For some reason the old element's handler won't fire after removed and added again.
+                e = $('<a href="#expand">...</a>');
+                e.click(expand);
+                n.append(e);
+                evt.stopPropagation();
+                evt.stopImmediatePropagation();
+                evt.preventDefault();
+              }
+              e.click(expand);
+              n.click(minimize);
+              n.text(mintext);
+              n.append(e)
+            }
+            return n;
           }
         }
         var elem = render(value);
@@ -1084,7 +1264,7 @@
           return false;
         }).prependTo($("a", li));
       }
-    }
+    },
 
   });
 
@@ -1093,7 +1273,7 @@
     for (var i=0; i < parts.length; i++) {
       encoded.push(encodeURIComponent(parts[i]));
     };
-    return encoded.join('/');
+    return encoded.join('%2f');
   }
 
 })(jQuery);
